# A GAP function to make the character table of (2 x 2.G ):2

t_g := CharacterTable("Fi22");
t_g2 := CharacterTable("Fi22.2");
t_2g := CharacterTable("2.Fi22");
t_2g2 := CharacterTable("2.Fi22.2");

# t_g := CharacterTable("A5");
# t_g2 := CharacterTable("A5.2");
# t_2g := CharacterTable("2.A5");
# t_2g2 := CharacterTable("2.A5.2");

# t_g := CharacterTable("L3(2)");
# t_g2 := CharacterTable("L3(2).2");
# t_2g := CharacterTable("2.L3(2)");
# t_2g2 := CharacterTable("2.L3(2).2");

# t_g := CharacterTable("A6");
# t_g2 := CharacterTable("S6");
# t_2g := CharacterTable("2.A6");
# t_2g2 := CharacterTable("2.S6");

# t_g := CharacterTable("F4(2)");
# t_g2 := CharacterTable("F4(2).2");
# t_2g := CharacterTable("2.F4(2)");
# t_2g2 := CharacterTable("2.F4(2).2");

# t_g := CharacterTable("Suz");
# t_g2 := CharacterTable("Suz.2");
# t_2g := CharacterTable("2.Suz");
# t_2g2 := CharacterTable("2.Suz.2");


# CharacterTableTwoSquaredGsplitTwo(t_g, t_g2, t_2g, t_2g2,1,1);


CharacterTableTwoSquaredGsplitTwo := function(t_g, t_g2, t_2g, t_2g2, proj1, proj2)
  local fus_g_g2, fus_2g_2g2, t_g2_ord, t_2g2_ord, t_g2_cen, proj_g, proj_g2,
   outer_g2, fused_g2, split_g, split_gx, split_g2, crossFused, g2_2g2_ord,
   classTypes, classLift, i, j, k, l, m, n, t,
   T, Torders, Tcentralisers, fusT, fus2T, posT, classNumber,
   mats, temp, r, jj, pfus, kr, kc, z,
   Tsizes, p, pmp, t_g2_p2, t_2g2_p2, pmptest, primes, t_g2_pmp, pm;
  # t_g is the character table of G
  # t_g2 is the character table of G:2
  # t_2g is the character table of 2'G
  # t_2g2 is the character table of 2'G:2
  # I have assumed that we have 2'G:2 and not
  # the isoclinic group 2'G'2
  # but I'm not sure if this matters
  
  fus_g_g2 := FusionConjugacyClasses(t_g,t_g2);
  fus_2g_2g2 := FusionConjugacyClasses(t_2g,t_2g2);
  
  #tab_g2_nam := ClassNames(tab_f2);
  t_g2_ord := OrdersClassRepresentatives(t_g2);
  t_2g2_ord := OrdersClassRepresentatives(t_2g2);
  
  t_g2_cen := SizesCentralizers(t_g2);
  
  if( GAPInfo.Version >= "4.4" ) then
    proj_g := ProjectivesInfo(t_g)[proj1];
  else
    proj_g := t_g!.projectives[proj2];
  fi;

  if( GAPInfo.Version >= "4.4" ) then
    proj_g2 := ProjectivesInfo(t_g2)[proj2];
  else
    proj_g2 := t_g2!.projectives[proj2];
  fi;

  # We have:
  #   G -----m1-----> G:2
  #   |               |
  #   |               |
  #   V               V
  #  2'G ----m2---> 2'G:2
  #   |               |
  #   |               |
  #   V               V
  # (2^2)'G ----> (2^2)'G:2

  
  # C L A S S   T Y P E S
  # The class types are explained in doc
  # doc  here description
  # 1ai  1    g ~/ zg in 2'G:2         "inner, no fuse"
  # 1aii 2    g ~/ gz  in 2'G but fuse "inner, fuse"
  # 1bi  3    g~gz in 2.G              "inner, non-split", same order
  # 1bii 12   g~gz in 2.G                                  double order
  # 2a   4    g ~ h ~ -g ~ -h          "complete fusion"
  # 2a   5    g~h                      "4x4"
  # 2b   6    g~zh                     "4x4 crossed"
  # 3a   7    outer, indiv 4, one class same order
  # 3b   8    outer, indiv 4, one class twice order
  # 3c   9    outer, indiv 4, two classes twice order
  # 3d   10   outer, div 4, same order
  # e3   11   outer, div 4, double order
  
  # Look at G & G:2 to obtain:
  # outer elts of G.2
  outer_g2 := Filtered( [1..Length(Irr(t_g2))] , i-> Position(fus_g_g2,i) = fail);
  # classes of G:2 that are 2 classes in G
  fused_g2 := Filtered( [1..Length(Irr(t_g2))] , i-> PositionNthOccurrence(fus_g_g2,i,2) <> fail); 
  
  # To find out about 2'G and 2'G:2 we need the prjective tables
  
  # classes of G which lift to only 1 class in 2'G
  # the .map goes from G to 2'G
  # so if class[i] lifts to one class j then
  #       class[i+1] lifts to class   j+1
  split_g := Filtered([1..Length(Irr(t_g))-1], i -> proj_g.map[i+1] <> proj_g.map[i]+1 );
  # Deal with the last class
  if  proj_g.map[ Length(Irr(t_g)) ]  <>  Length(Irr(t_g)) + Length(proj_g.chars)  then
    Add(split_g, Length(Irr(t_g)));
  fi;
  # we shall actually need this indexed by G:2
  split_gx := Filtered( [1..Length(Irr(t_g2))] , i -> Position(fus_g_g2, i) in split_g );
  
  # Now the same for G:2 and 2'G:2
  split_g2 := Filtered([1..Length(Irr(t_g2))-1], i -> proj_g2.map[i+1] <> proj_g2.map[i]+1 );
  # Deal with the last class
  if  proj_g2.map[ Length(Irr(t_g2)) ]  <>  Length(Irr(t_g2)) + Length(proj_g2.chars)  then
    Add(split_g2, Length(Irr(t_g2)));
  fi;
  
  # Detect cross-fused classes i.e., X ~ -Y and -X ~ Y
  # This won't work unless the classes are in the expected order.
  #                                                 fusion array looks like ...a b b a...        index by g2 
  crossFused := List( Filtered([1..Length(Irr(t_2g))-3], i -> fus_2g_2g2[i] = fus_2g_2g2[i+3] ), i ->  Position(proj_g2.map, fus_2g_2g2[i]) );
  
  # Orders that elts lift to G:2 to 2'G:2
  g2_2g2_ord := List( [1..Length(Irr(t_g2))] , i -> t_2g2_ord[ proj_g2.map[ i ] ] );

  
  # Now we can identify the class types
  classTypes := [];
  classLift := []; # no of new classes
  for i in [1..Length(Irr(t_g2))] do
    if not i in outer_g2 then                          # INNER
      if not i in fused_g2 then                        # CASE 1
        if i in split_gx then                          # CASE a
          # for i & ii, does g fuse with zg in 2'G:2 ?
          if i in split_g2 then # CASE i
            classTypes[i] := 1;
            classLift[i] := 3;
          else
            classTypes[i] := 2;
            classLift[i] := 3;
          fi;
        else                                           # CASE b
          if g2_2g2_ord[ i ] = t_g2_ord[i] then
            classTypes[i] := 3;
            classLift[i] := 2;
          else
            classTypes[i] := 12;
            classLift[i] := 2;
          fi;
        fi;
      else # 2 classes of Fi22 fuse                      CASE 2
        if i in split_gx then
          if i in crossFused then
            classTypes[i] := 6; 
            classLift[i] := 4;
          else
            classTypes[i] := 5;
            classLift[i] := 4;
          fi;
        else # g & zg conj, so case 2a
          classTypes[i] := 4;
          classLift[i] := 2;
        fi;
      fi;
    else # OUTER  
      if not t_g2_ord[i] / 4 in Integers then
        if not i in split_g2 then
          if g2_2g2_ord[ i ] = t_g2_ord[i] then
            classTypes[i] := 7;
          else
            classTypes[i] := 8;
          fi;
        else
          classTypes[i] := 9;
        fi;
      else
        if g2_2g2_ord[ i ] = t_g2_ord[i] then
          classTypes[i] := 10;
        else
          classTypes[i] := 11;
        fi;
      fi;
      classLift[i] := 2;
    fi;
  od;
  
          
  
  #  M A K E   T A B L E
  
  Torders := [];
  Tcentralisers := [];
  T := [];
  fusT := []; #lift position from G.2
  fus2T := []; #lift position from 2.G.2 (zg)
  posT := []; #start of block, going up  
  
  # Torders and Tcentralisers
  # if g has even order (in the quotient) then char tab of 2.Fi22.2 implies
  # that 2.g has the same order in both the split and non-split case
  # If g has odd order then the coset splits and the new things
  # have order 2|g|.
  # for outer elts, slap slap slap slap

  t := 1;
  for i in [1..Length(Irr(t_g2))] do
    posT[i] := t;
    if classTypes[i] in [1,2] then
      # 1 & 2
      # Can do both together because type 2 elts always have even order in g2
      Torders[t] := t_g2_ord[i];
      if t_g2_ord[i]/2 in Integers then
        Torders[t+1] := t_g2_ord[i];
        Torders[t+2] := t_g2_ord[i];
      else
        Torders[t+1] := t_g2_ord[i] * 2;
        Torders[t+2] := t_g2_ord[i] * 2;
      fi;
      Tcentralisers[t] := 4*t_g2_cen[i]/1;
      Tcentralisers[t+1] := 4*t_g2_cen[i]/2;
      Tcentralisers[t+2] := 4*t_g2_cen[i]/1;
      if classTypes[i] = 1 then
        fusT[i] := t;
        fus2T[i] := t+2;
      else
        fusT[i] := t+1;
        fus2T[i] := t+1;
      fi;
      t  := t+3;
    elif classTypes[i] = 3 then
      # 3. Even order
      Torders[t] := t_g2_ord[i];
      Torders[t+1] := t_g2_ord[i];
      Tcentralisers[t] := 2*t_g2_cen[i]/1;
      Tcentralisers[t+1] := 2*t_g2_cen[i]/1;
      fusT[i] := t;
      fus2T[i] := t;
      t := t+2;
    elif classTypes[i] = 12 then
      # 3. Even order
      Torders[t] := t_g2_ord[i] * 2;
      Torders[t+1] := t_g2_ord[i] * 2;
      Tcentralisers[t] := 2*t_g2_cen[i]/1;
      Tcentralisers[t+1] := 2*t_g2_cen[i]/1;
      fusT[i] := t;
      fus2T[i] := t;
      t := t+2;
    elif classTypes[i] = 4 then
      # 3. Even order
      Torders[t] := t_g2_ord[i];
      Torders[t+1] := t_g2_ord[i];
      Tcentralisers[t] := 2*t_g2_cen[i]/1;
      Tcentralisers[t+1] := 2*t_g2_cen[i]/1;
      fusT[i] := t;
      fus2T[i] := t;
      t := t+2;
    elif classTypes[i] in [5,6] then
      # type 5 always have even order
      Torders[t] := t_g2_ord[i];
      if t_g2_ord[i]/2 in Integers then
        Torders[t+1] := t_g2_ord[i];
        Torders[t+2] := t_g2_ord[i];
        Torders[t+3] := t_g2_ord[i];
      else
        Torders[t+1] := t_g2_ord[i] * 2;
        Torders[t+2] := t_g2_ord[i] * 2;
        Torders[t+3] := t_g2_ord[i] * 2;
      fi;
      Tcentralisers[t] := 4*t_g2_cen[i]/1;
      Tcentralisers[t+1] := 4*t_g2_cen[i]/1;
      Tcentralisers[t+2] := 4*t_g2_cen[i]/1;
      Tcentralisers[t+3] := 4*t_g2_cen[i]/1;
      if classTypes[i] = 5 then
        fusT[i] := t;
        fus2T[i] := t+3;
      else
        fusT[i] := t+1;
        fus2T[i] := t+2;
      fi;
      t := t+4;
    elif classTypes[i] = 7 then
      Torders[t] := t_g2_ord[i];
      Torders[t+1] := t_g2_ord[i] * 2;
      Tcentralisers[t] := 2*t_g2_cen[i]/1;
      Tcentralisers[t+1] := 2*t_g2_cen[i]/1;
      fusT[i] := t;
      fus2T[i] := t;
      t := t+2;
    elif classTypes[i] = 8 then
      Torders[t] := t_g2_ord[i] * 2;
      Torders[t+1] := t_g2_ord[i];
      Tcentralisers[t] := 2*t_g2_cen[i]/1;
      Tcentralisers[t+1] := 2*t_g2_cen[i]/1;
      fusT[i] := t;
      fus2T[i] := t;
      t := t+2;
    elif classTypes[i] = 9 then
      Torders[t] := t_g2_ord[i];
      Torders[t+1] := t_g2_ord[i] * 2;
      Tcentralisers[t] := 2*t_g2_cen[i]/1;
      Tcentralisers[t+1] := 2*t_g2_cen[i]/1;
      fusT[i] := t;
      fus2T[i] := t;
      t := t+2;
    elif classTypes[i] = 10 then
      Torders[t] := t_g2_ord[i];
      Torders[t+1] := t_g2_ord[i];
      Tcentralisers[t] := 2*t_g2_cen[i]/1;
      Tcentralisers[t+1] := 2*t_g2_cen[i]/1;
      fus2T[i] := t;
      fusT[i] := t;
      t := t+2;
    elif classTypes[i] = 11 then
      Torders[t] := t_g2_ord[i] * 2;
      Torders[t+1] := t_g2_ord[i] * 2;
      Tcentralisers[t] := 2*t_g2_cen[i]/1;
      Tcentralisers[t+1] := 2*t_g2_cen[i]/1;
      fus2T[i] := t;
      fusT[i] := t;
      t := t+2;
    fi;
  od;
  classNumber := t-1;
  
  unposT := []; 
  x := 1;
  for i in [1..classNumber] do
    y := Position(posT, i);
    if y <> fail then
      x := y;
    fi;
    unposT[i] := x;      
  od;
  
  for i in [1..classNumber] do
    T[i] := [];
  od;
  
  mats := [];
  mats[1] := [[1,1,1],[2,0,-2],[1,-1,1]];
  mats[2] := [[1,1,1],[2,0,-2],[-1,1,-1]];
  mats[3] := [[1,1],[1,-1]];
  mats[4] := [[1,1],[1,-1]];
  mats[5] := [[1,1,1,1],[1,-1,1,-1],[1,1,-1,-1],[1,-1,-1,1]];
  mats[6] := [[1,1,1,1],[1,-1,1,-1],[1,1,-1,-1],[-1,1,1,-1]];
  mats[7] := [[1,1],[1,-1]];
  mats[8] := [[1,1],[1,-1]];
  mats[9] := [[1,1],[1,-1]];
  mats[10] := [[1,1],[1,-1]];
  mats[11] := [[1,1],[1,-1]];
  mats[12] := [[1,1],[1,-1]];
  
  
  #  T A B L E
  
  for i in [1..Length(Irr(t_g2))] do
    temp := []; # table fragment
    r := 1;  # position down the chars of this chunk (row)
    
    # Section for Fi22.2 at the top
    for j in [1..Length(Irr(t_g2))] do
      temp[r] := [];
      for k in [1..classLift[i]] do
        temp[r][k] := Irr(t_g2)[j][i];
      od;
      r := r+1;
    od;
    
    # Section for projective characters of Fi22
    if classLift[i] = 4 then
      jj := 2;
    elif classLift[i] = 3 then
      jj := 1;
    else
      jj := 0;
    fi;
    
    # Find which cols of proj_g to use, inverse of fus_f_g2
    pfus := [];
    for k in [1..Length(fus_g_g2)] do
      if fus_g_g2[k] = i then
        Add(pfus, k);
      fi;
    od;
    
    # multiply the projective characters by the FCM and
    # store in temp starting in row r
    for kr in [1..Length(proj_g.chars)] do # rows of the projective char tab
      temp[r] := [];
      for kc in [1..classLift[i]] do # cols of the fcm
        z := 0;
        for j in [1..jj] do
          z := z + proj_g.chars[kr][pfus[j]] * mats[classTypes[i]][j+1][kc];
        od;
        temp[r][kc] := z;
      od;
      r := r+1;
    od;
    
    # section for the last Fi22.2
    for j in [1..Length(Irr(t_g2))] do
      temp[r] := [];
      for k in [1..classLift[i]] do
        temp[r][k] := Irr(t_g2)[j][i] * mats[classTypes[i]][classLift[i]][k];
      od;
      r := r+1;
    od;
    
    # copy temp into the char tab
    for j in [1..Length(temp)] do
      for k in [1..Length(temp[i])] do
        T[j][posT[i]+k-1] := temp[j][k];
      od;
    od;
  od;
  
  
  
  Tsizes := [];
  for i in [1..Length(Tcentralisers)] do
    Tsizes[i] := Tcentralisers[1]/Tcentralisers[i];
  od;
  
  # # # P O W E R   M A P S
  pmp := [];
  pmp[1] := [];
  for i in [1..classNumber] do
    pmp[1][i] := i;
  od;
  # We need the power map of 2.g.2 to work out the 2-power map
  t_g2_p2 := PowerMap(t_g2,2);
  t_2g2_p2 := PowerMap(t_2g2,2);
  pmp[2] := [];
  j := 1;
  for i in [1..Length(Irr(t_g2))] do
    if  classTypes[i] in [7,9]  then
      pmp[2][j] := fusT[ t_g2_p2[i] ];
      pmp[2][j+1] := fus2T[ t_g2_p2[i] ];
      j := j+2;
    elif classTypes[i] = 8 then
      #its the other way round
      pmp[2][j] := fus2T[ t_g2_p2[i] ];
      pmp[2][j+1] := fusT[ t_g2_p2[i] ];
      j := j+2;
    elif classTypes[i] in [10,11] then
      #must consult the map for 2.G.2
      m := Position( proj_g2.map, t_2g2_p2[ proj_g2.map[i]] );
      if m = fail then #they swap
        pmp[2][j]:= fus2T[ t_g2_p2[ i ] ];
        pmp[2][j+1] := fusT[ t_g2_p2[ i ] ];
      else
        pmp[2][j]:= fusT[ t_g2_p2[ i ] ];
        pmp[2][j+1] := fus2T[ t_g2_p2[ i ] ];
      fi;
      j := j+2;
    else #inner. Must consult the map for 2.G.2
      m := Position( proj_g2.map, t_2g2_p2[ proj_g2.map[i]] );
      if m = fail then
        n := fus2T[ t_g2_p2[ i ] ];
      else
        n := fusT[ t_g2_p2[ i ] ];
      fi;
      for k in [1..classLift[i]] do
        pmp[2][j] := n;
        j := j+1;
      od;
    fi;
  od;
  #  O D D   P O W E R   M A P S
  
    t := function()
    local t;
    t := rec();
    t.Irr := T;
    t.NrConjugacyClasses := Length(T);
    t.Size := Tcentralisers[1];
    t.OrdersClassRepresentatives := Torders;
    t.SizesCentralizers := Tcentralisers;
    t.SizesConjugacyClasses := Tsizes;
    t.UnderlyingCharacteristic := 0;
    t.Identifier := "(2x2.G):2";
    t.InfoText := "A group of shape (2x2.G):2";
    ConvertToCharacterTable(t);
    return t;
  end;
  t := t();

  pmptest := true;
  # The new things on the bottom are not effected by odd powering, so
  # 11g cubes to 11g^3,...
  primes := Unique(IntegerFactorization(Tcentralisers[1]));
  RemoveElmList(primes, 1); #get rid of 2
  for p in primes do
    pmp[p] := [];
    t_g2_pmp := PowerMap(t_g2,p);
    j := 1;
    for k in [1..Length(Irr(t_g2))] do
      pm := [ posT[ t_g2_pmp[k] ] ];
      for l in [1..classLift[k]-1] do
        Add(pm,pm[1]+l);
      od;
      for l in [1..classLift[k]] do
        pmp[p][j] := pm;
        j := j+1;
      od;
    od;
    #Print(p, ": ", Congruences(t,Irr(t),pmp[p],p,false)," ", ConsiderKernels(t, Irr(t), pmp[p], p, false),"\n");
    pmptest := pmptest and Congruences(t,Irr(t),pmp[p],p,false);
    pmptest := pmptest and ConsiderKernels(t, Irr(t), pmp[p], p, false);
  od;
  
  if not pmptest then
    Print("WARNING! Bad power maps! Sorry.\n");
  fi;
  
  t := function()
    local t;
    t := rec();
    t.Irr := T;
    t.NrConjugacyClasses := Length(T);
    t.Size := Tcentralisers[1];
    t.OrdersClassRepresentatives := Torders;
    t.SizesCentralizers := Tcentralisers;
    t.SizesConjugacyClasses := Tsizes;
    t.UnderlyingCharacteristic := 0;
    t.Identifier := "(2x2.G):2";
    t.InfoText := "A group of shape (2x2.G):2";
    t.ComputedPowerMaps := pmp;
    ConvertToCharacterTable(t);
    return t;
  end;
  t := t();

  return t;
end;



